package com.yaoyicloud.render.cso;

import java.io.IOException;
import java.util.Map;

import com.yaoyicloud.easier.common.file.core.FileProperties;
import com.yaoyicloud.easier.common.file.core.FileTemplate;
import org.apache.commons.collections4.MapUtils;

import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.protobuf.util.JsonFormat;
import com.yaoyicloud.config.CommonDataCache;
import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.factory.CSOAbstractRender;
import com.yaoyicloud.message.CSOProtos;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public final class EntProductIntroductionRender extends CSOAbstractRender {
    private final FilerepoProperties filerepoProperties;
    private final CommonDataCache commonDataCache;

    public EntProductIntroductionRender(String cwd, FileTemplate ossTemplate, FileProperties properties,
        FilerepoProperties filerepoProperties, CommonDataCache commonDataCache) {
        super(cwd, ossTemplate, properties);
        this.filerepoProperties = filerepoProperties;
        this.commonDataCache = commonDataCache;
    }

    @Override
    protected Map<String, Object> renderDocx(String info, String relationId, Integer counter) throws IOException {
        log.info("开始渲染CSO企业报告产品介绍模块，relationId: {}", relationId);

        CSOProtos.ProductIntroduction.Builder header = CSOProtos.ProductIntroduction.newBuilder();

        JsonFormat.parser().merge(info, header);
        CSOProtos.ProductIntroduction defaultInstance = CSOProtos.ProductIntroduction.getDefaultInstance();
        CSOProtos.ProductIntroduction mergedProto = defaultInstance.toBuilder()
            .mergeFrom(header.build())
            .build();

        String completeJson = JsonFormat.printer()
            .includingDefaultValueFields()
            .print(mergedProto);
        ObjectMapper objectMapper = new ObjectMapper();
        Map<String, Object> data = objectMapper.readValue(completeJson, new TypeReference<Map<String, Object>>() {});
        Map<String, Object> commonDataCacheData = commonDataCache.getData(relationId);
        if (MapUtils.isNotEmpty(commonDataCacheData)) {
            data.putAll(commonDataCacheData);
        }
        data.put("levelInteger", counter);
        fillDefaultValues(data);
        return data;
    }

    @Override
    protected ConfigureBuilder builder(Map<String, Object> data) {
        ConfigureBuilder builder = Configure.builder();
        builder.bind("drugDetails", this.indicatorsRenderPolicyToProtobuf());
        return builder;
    }

    @Override
    protected String getBasicPath() throws IOException {
        return filerepoProperties.getBasePath();
    }

    @Override
    protected String getReportImagePath() {
        return filerepoProperties.getReportImagePath();
    }

    /**
     * 填充默认值，确保所有必要字段都存在
     */
    private void fillDefaultValues(Map<String, Object> data) {
        processJsonData(data, "drugDetails");

    }

}
