package com.yaoyicloud.render.platform.update;

import java.io.IOException;

import java.util.Map;
import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.policy.RenderPolicy;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.protobuf.util.JsonFormat;
import com.yaoyicloud.config.CommonDataCache;
import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.factory.FXYAbstractRender;
import com.yaoyicloud.message.FxyProtos;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;

@Slf4j
public final class AntiBriberyNewRender extends FXYAbstractRender {
    private final FilerepoProperties filerepoProperties;
    private final CommonDataCache commonDataCache;

    // 注入父类所需的 cwd 参数
    public AntiBriberyNewRender(String cwd, FilerepoProperties filerepoProperties, CommonDataCache commonDataCache) {
        super(cwd);
        this.filerepoProperties = filerepoProperties;
        this.commonDataCache = commonDataCache;
    }

    @Override
    protected String getBasicPath() throws IOException {
        return filerepoProperties.getBasePath();
    }

    @Override
    protected String getReportImagePath() {
        return filerepoProperties.getReportImagePath();
    }

    @Override
    protected Map<String, Object> renderDocx(String info, String relationId, Integer counter) throws IOException {
        log.info("开始渲染其他风险报告模块，relationId: {}", relationId);

        // 通过默认protobuf实例来填充不存在的key
        FxyProtos.AntiBribery.Builder basicInfoBuilder = FxyProtos.AntiBribery.newBuilder();
        JsonFormat.parser().merge(info, basicInfoBuilder);

        FxyProtos.AntiBribery defaultInstance = FxyProtos.AntiBribery.getDefaultInstance();
        FxyProtos.AntiBribery mergedProto = defaultInstance.toBuilder()
                .mergeFrom(basicInfoBuilder.build())
                .build();

        String completeJson = JsonFormat.printer()
                .includingDefaultValueFields()
                .print(mergedProto);
        ObjectMapper objectMapper = new ObjectMapper();
        Map<String, Object> data = objectMapper.readValue(completeJson, new TypeReference<Map<String, Object>>() {});
        Map<String, Object> commonDataCacheData = commonDataCache.getData(relationId);
        if (MapUtils.isNotEmpty(commonDataCacheData)) {
            data.putAll(commonDataCacheData);
        }
        fillDefaultValues(data);
        data.put("levelInteger", counter);
        // Map<String, Object> antiBriberySummary = (Map<String, Object>) data.get("antiBriberySummary");
        // String riskSummary = (String) antiBriberySummary.get("riskSummary");
        // antiBriberySummary.put("riskSummary", riskSummary == null || riskSummary.isEmpty() ? "-" : riskSummary);
        return data;
    }

    @Override
    protected ConfigureBuilder builder() {
        // 配置POI-TL渲染器
        ConfigureBuilder builder = Configure.builder();
        RenderPolicy indicatorsRenderPolicy = this.indicatorsRenderPolicyToProtobuf();
        builder.bind("questionnaireItems", indicatorsRenderPolicy);
        builder.bind("otherRiskChecks", indicatorsRenderPolicy);
        builder.useSpringEL();
        return builder;
    }

    /**
     * 填充默认值，确保所有必要字段都存在
     */
    private void fillDefaultValues(Map<String, Object> data) {
        Map<String, Object> antiBriberySummary = (Map<String, Object>) data.get("antiBriberySummary");
        antiBriberySummary.replaceAll((k, v) -> v.equals("") ? "-" : v);

    }



}
