package com.yaoyicloud.render.platform.update;

import java.io.IOException;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import com.yaoyicloud.config.CommonDataCache;
import com.yaoyicloud.factory.FXYAbstractRender;
import com.yaoyicloud.message.FxyProtos;
import org.apache.commons.collections4.CollectionUtils;

import com.deepoove.poi.config.Configure;
import com.deepoove.poi.config.ConfigureBuilder;
import com.deepoove.poi.policy.RenderPolicy;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.protobuf.util.JsonFormat;
import com.yaoyicloud.config.FilerepoProperties;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;

@Slf4j
public final class TaxNewRender extends FXYAbstractRender {
    private final FilerepoProperties filerepoProperties;
    private final CommonDataCache commonDataCache;

    public TaxNewRender(String cwd, FilerepoProperties filerepoProperties, CommonDataCache commonDataCache) {
        super(cwd);
        this.filerepoProperties = filerepoProperties;
        this.commonDataCache = commonDataCache;
    }

    @Override
    protected Map<String, Object> renderDocx(String info, String relationId, Integer counter) throws IOException {
        log.info("开始渲染税务风险模块，relationId: {}", relationId);


        FxyProtos.PublicRecord.Builder publicRecordBuilder = FxyProtos.PublicRecord.newBuilder();
        JsonFormat.parser().merge(info, publicRecordBuilder);
        FxyProtos.PublicRecord defaultInstance = FxyProtos.PublicRecord.getDefaultInstance();
        FxyProtos.PublicRecord mergedProto = defaultInstance.toBuilder()
                .mergeFrom(publicRecordBuilder.build())
                .build();

        String completeJson = JsonFormat.printer()
                .includingDefaultValueFields()
                .print(mergedProto);
        ObjectMapper objectMapper = new ObjectMapper();
        Map<String, Object> data = objectMapper.readValue(completeJson, new TypeReference<Map<String, Object>>() {});
        data.replaceAll((k, v) -> v.equals("") ? "-" : v);

        Map<String, Object> commonDataCacheData = commonDataCache.getData(relationId);
        if (MapUtils.isNotEmpty(commonDataCacheData)) {
            data.putAll(commonDataCacheData);
        }

        fillBasicDefaultValues(data);
        data.put("levelInteger", counter);
        // Map<String, Object> publicRecordSummary = (Map<String, Object>) data.get("publicRecordSummary");
        // publicRecordSummary.putIfAbsent("riskSummary", "-");
        // publicRecordSummary.putIfAbsent("suggestion", "");
        return data;
    }

    @Override
    protected ConfigureBuilder builder() {
        // 配置POI-TL渲染器
        ConfigureBuilder builder = Configure.builder();
        RenderPolicy indicatorsRenderPolicy = this.indicatorsRenderPolicyToProtobuf();

        builder.bind("taxationRiskChecks", indicatorsRenderPolicy);
        builder.bind("taxPenalties", indicatorsRenderPolicy);
        builder.useSpringEL();
        return builder;
    }

    @Override
    protected String getBasicPath() throws IOException {
        return filerepoProperties.getBasePath();
    }

    @Override
    protected String getReportImagePath() {
        return filerepoProperties.getReportImagePath();
    }


    /**
     * 填充默认值，确保所有必要字段都存在
     */
    private void fillBasicDefaultValues(Map<String, Object> data) {
        Map<String, Object> publicRecordSummary = (Map<String, Object>) data.get("publicRecordSummary");
        publicRecordSummary.replaceAll((k, v) -> v.equals("") ? "-" : v);

        List<Map<String, Object>> dishonestPersons = (List<Map<String, Object>>) data.get("dishonestPersons");
        if (CollectionUtils.isEmpty(dishonestPersons)) {
            Map<String, Object> defaultDishonestPerson = new HashMap<>();
            defaultDishonestPerson.put("id", "1");
            List<Map<String, Object>> defaultDishonestPersonList = Collections.singletonList(defaultDishonestPerson);
            data.put("dishonestPersons", defaultDishonestPersonList);

        }

        List<Map<String, Object>> severeViolations = (List<Map<String, Object>>) data.get("severeViolations");
        if (CollectionUtils.isEmpty(severeViolations)) {
            Map<String, Object> defaultSevereViolation = new HashMap<>();
            defaultSevereViolation.put("id", "1");
            List<Map<String, Object>> defaultSevereViolationList = Collections.singletonList(defaultSevereViolation);
            data.put("severeViolations", defaultSevereViolationList);
        }
    }
}
