package com.yaoyicloud.service.impl;

import static com.yaoyicloud.config.SessionInterceptor.SESSION_MAP;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;

import com.amazonaws.services.s3.model.S3Object;
import com.amazonaws.services.s3.model.S3ObjectInputStream;
import com.yaoyicloud.config.RelationCounterRedisUtil;
import com.yaoyicloud.easier.common.file.core.FileProperties;
import com.yaoyicloud.easier.common.file.core.FileTemplate;
import com.yaoyicloud.render.cso.EntAppendixRender;
import com.yaoyicloud.render.cso.EntProductIntroductionRender;
import com.yaoyicloud.render.cso.EntPromotionDetailsRender;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.springframework.stereotype.Service;
import com.yaoyicloud.config.CommonDataCache;
import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.config.ReportPathManager;
import com.yaoyicloud.constant.enums.ModuleType;
import com.yaoyicloud.entity.ReportGenerationResult;
import com.yaoyicloud.render.cso.EntHeaderSectionRender;
import com.yaoyicloud.render.cso.EntPromotionSummaryRender;
import com.yaoyicloud.service.CsoReportService;
import com.yaoyicloud.tools.DocxUtil;

import cn.hutool.core.util.IdUtil;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Service
@RequiredArgsConstructor
public class CsoReportServiceImpl implements CsoReportService {
    private final FilerepoProperties filerepoProperties;
    private final ReportPathManager reportPathManager;
    private final CommonDataCache commonDataCache;
    private final FileTemplate ossTemplate;
    private final FileProperties properties;
    private final RelationCounterRedisUtil relationCounterRedisUtil;
    private final FileProperties fileProperties;

    /**
     * 处理单个模块的报告生成
     */
    @SuppressWarnings("checkstyle:ParameterNumber")
    private String processModule(ModuleType moduleType, String data,
        InputStream inputStream, String sessionId, Long relationId, Integer counter) throws Exception {
        String reportPath;
        String type = moduleType.getType();
        switch (moduleType) {
            case CSO_HEADER_XINHUA:
            case CSO_HEADER:
                reportPath = new EntHeaderSectionRender(sessionId, ossTemplate, fileProperties, filerepoProperties,
                    commonDataCache)
                        .createCSODocument(data, inputStream, String.valueOf(relationId), counter, type);
                break;
            case CSO_PRODUCT_INTRODUCTION_XINHUA:
            case CSO_PRODUCT_INTRODUCTION:
                reportPath = new EntProductIntroductionRender(sessionId, ossTemplate, fileProperties,
                    filerepoProperties, commonDataCache)
                        .createCSODocument(data, inputStream, String.valueOf(relationId), counter, type);
                break;
            case CSO_PROMOTION_SUMMARY:
                reportPath = new EntPromotionSummaryRender(sessionId, ossTemplate, fileProperties, filerepoProperties,
                    commonDataCache)
                        .createCSODocument(data, inputStream, String.valueOf(relationId), counter, type);
                break;
            case CSO_ATTACHMENT_XINHUA:
            case CSO_PROMOTION_DETAILS_XINHUA:
            case CSO_PROMOTION_DETAILS_1611890606:
            case CSO_PROMOTION_DETAILS:
                reportPath = new EntPromotionDetailsRender(sessionId, ossTemplate, fileProperties, filerepoProperties,
                    commonDataCache)
                        .createCSODocument(data, inputStream, String.valueOf(relationId), counter, type);
                break;

            case CSO_SITUATION_OVERVIEW_XINHUA:
            case CSO_PROMOTION_SIGNIFICANCE_XINHUA:
            case CSO_PROMOTION_SIGNIFICANCE_DRUG:
            case CSO_SITUATION_OVERVIEW_DRUG:
            case CSO_APPENDIX:
                reportPath =
                    new EntAppendixRender(sessionId, ossTemplate, fileProperties, filerepoProperties, commonDataCache)
                        .createCSODocument(data, inputStream, String.valueOf(relationId), counter, type);
                break;
            default:
                throw new UnsupportedOperationException("Unsupported module type: " + moduleType);
        }
        return reportPath;
    }

    @Override
    public ReportGenerationResult createCsoCheckReport(String data, Long relationId,
        ModuleType moduleType, Boolean mergeflag) throws Exception {
        // oss 获取文件
        S3Object s3Object = ossTemplate.getObject(properties.getBucketName(), moduleType.getType() + ".docx");
        S3ObjectInputStream objectContent = s3Object.getObjectContent();

        Integer count = relationCounterRedisUtil.getCount(relationId);
        String sessionId = SESSION_MAP.get(relationId.toString());

        String reportPath = processModule(moduleType, data, objectContent, sessionId, relationId, count);
        // 处理指定模块
        reportPathManager.addReportPath(sessionId, reportPath);
        relationCounterRedisUtil.increment(relationId, 1);
        String mergedReportPath = null;
        if (mergeflag) {
            List<String> reportPaths = reportPathManager.removeSession(sessionId);
            XWPFDocument targetDoc;
            try (FileInputStream fis = new FileInputStream(reportPaths.get(0))) {
                targetDoc = new XWPFDocument(fis);
            }
            List<XWPFDocument> sourceDocs = new ArrayList<>();
            for (int i = 1; i < reportPaths.size(); i++) {
                try (FileInputStream fis = new FileInputStream(reportPaths.get(i))) {
                    sourceDocs.add(new XWPFDocument(fis));
                }
            }
            String label = relationId + "_" + IdUtil.fastSimpleUUID();
            mergedReportPath = filerepoProperties.getBasePath() + "/" + label + ".docx";
            // 调用合并方法
            DocxUtil.mergeDocx(targetDoc, sourceDocs, mergedReportPath, relationId);
        }

        return new ReportGenerationResult(reportPath, mergedReportPath, mergeflag);
    }
}
