package com.yaoyicloud.service.impl;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import cn.hutool.core.util.IdUtil;
import com.amazonaws.services.s3.model.S3Object;
import com.amazonaws.services.s3.model.S3ObjectInputStream;
import com.yaoyicloud.config.CommonDataCache;
import com.yaoyicloud.config.FilerepoProperties;
import com.yaoyicloud.config.RelationCounterRedisUtil;
import com.yaoyicloud.config.ReportPathManager;
import com.yaoyicloud.constant.enums.ModuleType;
import com.yaoyicloud.easier.common.file.core.FileProperties;
import com.yaoyicloud.easier.common.file.core.FileTemplate;
import com.yaoyicloud.entity.ReportGenerationResult;

import com.yaoyicloud.render.ServiceProviderInfoRender;

import com.yaoyicloud.render.platform.AttachmentSectionRender;

import com.yaoyicloud.render.platform.update.AntiBriberyNewRender;
import com.yaoyicloud.render.platform.update.FXYInfoNewRender;
import com.yaoyicloud.render.platform.update.FinancialInfoNewRender;
import com.yaoyicloud.render.platform.update.PublicRecordNewRender;
import com.yaoyicloud.render.platform.update.TaxNewRender;
import com.yaoyicloud.service.ReportUpdateService;
import com.yaoyicloud.tools.DocxUtil;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.springframework.stereotype.Service;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;

import static com.yaoyicloud.config.SessionInterceptor.SESSION_MAP;

@Slf4j
@Service
@RequiredArgsConstructor
public class FxyReportUpdateServiceImpl implements ReportUpdateService {

    private final FilerepoProperties filerepoProperties;
    private final ReportPathManager reportPathManager;
    private final CommonDataCache commonDataCache;
    private final RelationCounterRedisUtil relationCounterRedisUtil;
    private final FileTemplate ossTemplate;
    private final FileProperties properties;

    @SuppressWarnings("checkstyle:ReturnCount")
    @Override
    public ReportGenerationResult createPlusVersionCheckReport(String data, Long relationId, ModuleType moduleType)
        throws Exception {

        // oss 获取文件
        S3Object s3Object = ossTemplate.getObject(properties.getBucketName(), moduleType.getType() + ".docx");
        S3ObjectInputStream objectContent = s3Object.getObjectContent();

        Integer count = relationCounterRedisUtil.getCount(relationId);
        String sessionId = SESSION_MAP.get(relationId.toString());
        // 处理指定模块
        String reportPath = processModule(moduleType, data, objectContent, sessionId, relationId, count);

        reportPathManager.addReportPath(sessionId, reportPath);
        relationCounterRedisUtil.increment(relationId, 1);

        String mergedReportPath = null;
        boolean isLastModule = reportPath.contains("attachmentSection");
        if (isLastModule) {
            List<String> reportPaths = reportPathManager.removeSession(sessionId);
            XWPFDocument targetDoc;
            try (FileInputStream fis = new FileInputStream(reportPaths.get(0))) {
                targetDoc = new XWPFDocument(fis);
            }
            List<XWPFDocument> sourceDocs = new ArrayList<>();
            for (int i = 1; i < reportPaths.size(); i++) {
                try (FileInputStream fis = new FileInputStream(reportPaths.get(i))) {
                    sourceDocs.add(new XWPFDocument(fis));
                }
            }
            String label = relationId + "_" + IdUtil.fastSimpleUUID();
            mergedReportPath = filerepoProperties.getBasePath() + "/" + label + ".docx";
            // 调用合并方法
            DocxUtil.mergeDocx(targetDoc, sourceDocs, mergedReportPath, relationId);

        }

        return new ReportGenerationResult(reportPath, mergedReportPath, isLastModule);
    }

    /**
     * 处理单个模块的报告生成
     */
    @SuppressWarnings("checkstyle:ParameterNumber")
    private String processModule(ModuleType moduleType, String data,
       InputStream inputStream, String sessionId, Long relationId, Integer counter) throws Exception {
        String reportPath = null;
        String type = moduleType.getType();
        switch (moduleType) {
//            case ANTIBRIBERY:
//                reportPath = new AntiBriberyRender(sessionId, filerepoProperties, commonDataCache)
//                    .createFXYDocument(data, inputStream, String.valueOf(relationId));
//                break;
            case PLATFORM_COMPANY_ANTIBRIBERY_NEW:
                reportPath = new AntiBriberyNewRender(sessionId, filerepoProperties, commonDataCache)
                    .createFXYDocument(data, inputStream, String.valueOf(relationId), counter, type);
                break;
//            case PLATFORM_COMPANY_BASICINFO:
//                reportPath = new BasicInfoRender(sessionId, filerepoProperties, commonDataCache)
//                    .createFXYDocument(data, inputStream, String.valueOf(relationId));
//                break;
            case PLATFORM_COMPANY_ATTACHMENTSECTION:
                reportPath = new AttachmentSectionRender(sessionId, filerepoProperties, commonDataCache)
                    .createFXYDocument(data, inputStream, String.valueOf(relationId), counter, type);
                break;
//            case ASSOCIATION_ATTACHMENTSECTION:
//                reportPath = new AssociationAttachmentSectionRender(sessionId, filerepoProperties, commonDataCache)
//                    .createFXYDocument(data, inputStream, String.valueOf(relationId));
//                break;
//            case FOUNDATION_ATTACHMENTSECTION:
//                reportPath = new FoundationAttachmentSectionRender(sessionId, filerepoProperties, commonDataCache)
//                    .createFXYDocument(data, inputStream, String.valueOf(relationId));
//                break;
            case COMMON:
                reportPath = new ServiceProviderInfoRender(sessionId, filerepoProperties, commonDataCache)
                    .createFXYDocument(data, inputStream, String.valueOf(relationId), counter, type);
                break;
//            case PLATFORM_COMPANY_FINANCIALINFO:
//                reportPath = new FinancialInfoRender(sessionId, filerepoProperties, commonDataCache)
//                    .createFXYDocument(data, inputStream, String.valueOf(relationId));
//                break;
            case PLATFORM_COMPANY_FINANCIALINFO_NEW:
                reportPath = new FinancialInfoNewRender(sessionId, filerepoProperties, commonDataCache)
                    .createFXYDocument(data, inputStream, String.valueOf(relationId), counter, type);
                break;
//            case FOUNDATION_FINANCIALINFO:
//                reportPath = new FoundationfinancialInfoRender(sessionId, filerepoProperties, commonDataCache)
//                    .createFXYDocument(data, inputStream, String.valueOf(relationId));
//                break;
//            case PUBLIC_RECORD:
//                reportPath = new PublicRecordRender(sessionId, filerepoProperties, commonDataCache)
//                    .createFXYDocument(data, inputStream, String.valueOf(relationId));
//                break;
            case PLATFORM_COMPANY_PUBLIC_RECORD_NEW:
                reportPath = new PublicRecordNewRender(sessionId, filerepoProperties, commonDataCache)
                    .createFXYDocument(data, inputStream, String.valueOf(relationId), counter, type);
                break;
            case PLATFORM_COMPANY_BASICINFO_NEW:
                reportPath = new FXYInfoNewRender(sessionId, filerepoProperties, commonDataCache)
                    .createFXYDocument(data, inputStream, String.valueOf(relationId), counter, type);
                break;
//            case FOUNDATION_BASICINFO:
//                reportPath = new FoundationBasicInfoRender(sessionId, filerepoProperties, commonDataCache)
//                    .createFXYDocument(data, inputStream, String.valueOf(relationId));
//                break;
//            case ASSOCIATION_BASICINFO:
//                reportPath = new AssociationBasicInfoRender(sessionId, filerepoProperties, commonDataCache)
//                    .createFXYDocument(data, inputStream, String.valueOf(relationId));
//                break;
//            case PROJECT:
//                reportPath = new ProjectInfoRender(sessionId, filerepoProperties, commonDataCache)
//                    .createFXYDocument(data, inputStream, String.valueOf(relationId));
//                break;
//            case INTERESTCONFLICTS:
//                reportPath = new InterestConflictsRender(sessionId, filerepoProperties, commonDataCache)
//                    .createFXYDocument(data, inputStream, String.valueOf(relationId));
//                break;
            case PLATFORM_COMPANY_TAX_NEW:
                reportPath = new TaxNewRender(sessionId, filerepoProperties, commonDataCache)
                    .createFXYDocument(data, inputStream, String.valueOf(relationId), counter, type);
                break;
            default:
                throw new UnsupportedOperationException("Unsupported module type: " + moduleType);
        }
        return reportPath;
    }

}
